// #include <MPU6050.h>
#include <MPU6050_6Axis_MotionApps20.h>
// #include <MPU6050_6Axis_MotionApps612.h>
// #include <MPU6050_9Axis_MotionApps41.h>
#include <helper_3dmath.h>


#include "I2Cdev.h"
#include <SPI.h>

// #include "MPU6050_6Axis_MotionApps20.h"

#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
#include "Wire.h"
#endif

#define OUTPUT_OVER_SERIAL
// #define OUTPUT_OVER_RADIO


MPU6050 mpu;

// quaternion components in a [w, x, y, z] format (not best for parsing
// on a remote host such as Processing or something though)
#define OUTPUT_READABLE_QUATERNION

#define LED_PIN LED_BUILTIN
bool blinkState = false;
int blinkTimer = 0;

// MPU control/status vars
bool dmpReady = false;   // set true if DMP init was successful
uint8_t mpuIntStatus;    // holds actual interrupt status byte from MPU
uint8_t devStatus;       // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;     // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;      // count of all bytes currently in FIFO
uint8_t fifoBuffer[64];  // FIFO storage buffer

// orientation/motion vars
Quaternion q;         // [w, x, y, z]         quaternion container
VectorInt16 aa;       // [x, y, z]            accel sensor measurements
VectorInt16 aaReal;   // [x, y, z]            gravity-free accel sensor measurements
VectorInt16 aaWorld;  // [x, y, z]            world-frame accel sensor measurements
VectorFloat gravity;  // [x, y, z]            gravity vector
float euler[3];       // [psi, theta, phi]    Euler angle container
float ypr[3];         // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector

// unused
uint8_t teapotPacket[14] = { '$', 0x02, 0, 0, 0, 0, 0, 0, 0, 0, 0x00, 0x00, '\r', '\n' };

// Sage
void readFifoBuffer_() {
  mpu.resetFIFO();
  fifoCount = mpu.getFIFOCount();
  while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();
  mpu.getFIFOBytes(fifoBuffer, packetSize);
}

void setup() {
// GYRO SETUP

// join I2C bus (I2Cdev library doesn't do this automatically)
#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
  Wire.begin();
  Wire.setClock(400000);  // 400kHz I2C clock. Comment this line if having compilation difficulties
#elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
  Fastwire::setup(400, true);
#endif
  #ifdef OUTPUT_OVER_SERIAL
    Serial.begin(57600);
  #endif

  // while (!Serial);

  Serial.println(F("Initializing I2C devices..."));
  mpu.initialize();
  // pinMode(INTERRUPT_PIN, INPUT);

  // verify connection
  Serial.println(F("Testing device connections..."));
  Serial.println(mpu.testConnection() ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));

  // load and configure the DMP
  Serial.println(F("Initializing DMP..."));
  devStatus = mpu.dmpInitialize();

  // offsets here, scaled for min sensitivity
  // 905	1578	1445	117	53	19
  mpu.setXGyroOffset(117);
  mpu.setYGyroOffset(53);
  mpu.setZGyroOffset(19);
  mpu.setZAccelOffset(1788);  // 1688 factory default for my test chip

  // make sure it worked (returns 0 if so)
  if (devStatus == 0) {
    // Calibration Time: generate offsets and calibrate our MPU6050
    mpu.CalibrateAccel(15);
    mpu.CalibrateGyro(15);
    mpu.PrintActiveOffsets();
    // turn on the DMP, now that it's ready
    Serial.println(F("Enabling DMP..."));
    mpu.setDMPEnabled(true);

    Serial.println(F("DMP ready! Waiting for first interrupt..."));
    dmpReady = true;

    // get expected DMP packet size for later comparison
    packetSize = mpu.dmpGetFIFOPacketSize();
  } else {
    // ERROR!
    // 1 = initial memory load failed
    // 2 = DMP configuration updates failed
    // (if it's going to break, usually the code will be 1)
    Serial.print(F("DMP Initialization failed (code "));
    Serial.print(devStatus);
    Serial.println(F(")"));
  }
  pinMode(6, OUTPUT);
  pinMode(4, OUTPUT);
  // configure LED for output
 
}

int loop_count = 0;
// Sage
void loop() {


  // if programming failed, don't try to do anything
  if (!dmpReady) return;

  // blink LED to indicate activity  // Sage 3/24
  if (blinkTimer / 5) {
    blinkTimer = 0;
    //digitalWrite(LED_PIN, blinkState);
    blinkState = !blinkState;
  }
  blinkTimer++;

  readFifoBuffer_();  // Sage 3/24

  String msg = "";

#ifdef OUTPUT_READABLE_QUATERNION
  // display quaternion values in easy matrix form: w x y z
  mpu.dmpGetQuaternion(&q, fifoBuffer);
  msg = String(q.w) + " " + String(q.x) + " " + String(q.y) + " " + String(q.z);
#endif
  // loop_count ++;
  // msg = "Hello World";
  // msg = String(loop_count);
#ifdef OUTPUT_OVER_SERIAL
  Serial.println(msg);
#endif
  delay(10000);

if (q.w > 0.8 && q.w < 0.9) {
  digitalWrite(6, HIGH);  
  digitalWrite(2, HIGH);
  delay(100);     
  digitalWrite(2, LOW);
  digitalWrite(6, LOW);
}   

if (q.w > 0.55 && q.w < 0.75 && q.x > -0.6 && q.x < -0.8 && q.y > -0.3 && q.y < 0.3) {
   digitalWrite(4, HIGH); 
   digitalWrite(2, HIGH);
  delay(100);     
  digitalWrite(4, LOW);
  digitalWrite(2, LOW);
}

if (q.w < .05 && q.w > -0.1 && q.x > -1.0 && q.x < -0.5 && q.z > -0.1 && q.z < 0.1) {
  digitalWrite(2, HIGH);  
  delay(100);     
  digitalWrite(2, LOW);
} 
}

          
